#
#   Copyright 2011 Michael Phan-Ba
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

request = require 'request'


exports.Mixpanel = class Mixpanel

  request: request
  server: 'http://api.mixpanel.com'
  alias: true
  autoTime: false


  constructor: (options) ->
    @token = options.token

    @request = options.request if options.request?
    @endpoint = options.endpoint if options.endpoint?

    @ip = !!options.ip if options.ip?
    @test = !!options.test if options.test?
    @alias = !!options.alias if options.alias?
    @autoNow = !!options.autoNow if options.autoNow?


  track: (event) ->

    # unpack arguments
    args = Array.prototype.slice.call arguments, 1
    callback = args.pop() if typeof args[args.length - 1] is 'function'
    properties = args.shift() or {}
    options = args[0] or {}

    # flags
    setIp = if options.ip? then options.ip else @ip
    setTime = if options.autoNow? then options.autoNow else @autoNow
    test = if options.test? then options.test else @test

    # set required token property
    properties.token = @token

    # set time
    if setTime
      properties.time = parseInt Date.now() / 1000
    else if properties.time?.getTime
      properties.time = parseInt properties.time.getTime() / 1000

    # rename aliased properties
    if @alias
      if properties.id?
        properties.distinct_id ?= properties.id
        delete properties.id
      if properties.tag?
        properties.mp_name_tag ?= properties.tag
        delete properties.tag

    # remove ip when automatically set
    delete properties.ip if setIp

    # base64 json data
    data = event: event, properties: properties
    data = new Buffer(JSON.stringify data).toString 'base64'

    # build query
    query = [ 'data=' + data ]
    query.push 'ip=1' if setIp
    query.push 'test=1' if test
    query = query.join '&'

    # build uri
    uri = @server + '/track?' + query

    # send request
    if callback
      @request uri, (err, res, body) ->
        return callback err if err
        callback null, body is '1'

    else
      @request uri
