#
#   Copyright 2011 Michael Phan-Ba
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

assert      = require 'assert'
sinon       = require 'sinon'
url         = require 'url'
querystring = require 'querystring'


mixpanel = require '../lib/index'


describe 'Mixpanel', ->
  live = process.env.MIXPANEL_TOKEN?

  if live
    token = process.env.MIXPANEL_TOKEN
    request = sinon.spy require 'request'

  else
    token = 'test'
    request = sinon.stub()


  mp = mixpanel.createClient token: token, request: request, test: true
  mpServer = url.parse mp.server


  it 'should set token option', ->
    assert.equal mp.token, token

  it 'should set request option', ->
    assert.strictEqual mp.request, request

  it 'should set test option', ->
    assert mp.test


  track = (event, properties, options, callback) ->
    args = [ JSON.stringify event ]
    args.push JSON.stringify properties if properties or options
    args.push JSON.stringify options if options
    args.push 'fn' if callback
    args = args.join ', '

    describe "#track(#{args})", ->
      calledOnce = false
      requestUri = null

      it 'should succeed', (done) ->
        props = {}
        props[k] = v for k, v of properties if properties

        opts = {}
        opts[k] = v for k, v of options if options

        args = [ event ]
        args.push props if properties or options
        args.push opts if options

        if callback
          args.push (err, success) ->
            assert.ifError err
            assert.equal success, '1'
            done()

        unless live
          mp.request = sinon.stub()
          mp.request.yields null, {}, '1' if callback

        mp.track.apply mp, args

        calledOnce = mp.request.calledOnce
        requestUri = mp.request.args[0][0]

        mp.request.reset()

        properties ?= {}
        options ?= {}

        done() unless callback


      it 'should be called once', ->
        assert calledOnce

      it 'should use tracking URI', ->
        uri = url.parse requestUri

        assert.equal uri.protocol, mpServer.protocol
        assert.equal uri.host, mpServer.host
        assert.equal uri.port, mpServer.port
        assert.equal uri.pathname, '/track'

        query = querystring.parse uri.query

        assert.equal query.test, '1'
        assert.equal query.ip, '1' if options.ip

        data = JSON.parse new Buffer query.data, 'base64'

        properties.token = token

        delete properties.ip if options.ip

        if options.autoNow
          assert data.properties.time > 0
          delete data.properties.time
          delete properties.time

        assert.equal data.event, event
        assert.deepEqual data.properties, properties


  track 'event'
  track 'event', custom: true
  track 'event', null, ip: true
  track 'event', { custom: true, time: 0 }, { autoNow: true }
  track 'event', { custom: true }, { ip: true }

  track 'event', null, null, true
  track 'event', custom: true, null, true
  track 'event', null, ip: true, true
  track 'event', { custom: true, time: 0 }, { autoNow: true }, true
  track 'event', { custom: true }, { ip: true }, true
